// Copyright (c) 2016 The SigViewer Development Team
// Licensed under the GNU General Public License (GPL)
// https://www.gnu.org/licenses/gpl


#ifndef SIGNAL_GRID_GRAPHICS_ITEM_H
#define SIGNAL_GRID_GRAPHICS_ITEM_H

namespace sigviewer
{

class SignalGridGraphicsItem
{
public:
    SignalGridGraphicsItem();
};

}

#endif // SIGNAL_GRID_GRAPHICS_ITEM_H
