// Copyright (c) 2016 The SigViewer Development Team
// Licensed under the GNU General Public License (GPL)
// https://www.gnu.org/licenses/gpl


#ifndef SIGNAL_BROWSER_MODES_H
#define SIGNAL_BROWSER_MODES_H

namespace sigviewer
{

enum SignalVisualisationMode
{
    MODE_NEW,
    MODE_POINTER,
    MODE_HAND,
    MODE_VIEW_OPTIONS,
    MODE_INFO
};

}

#endif
