// Copyright (c) 2016 The SigViewer Development Team
// Licensed under the GNU General Public License (GPL)
// https://www.gnu.org/licenses/gpl


#ifndef APPLICATION_STATES_H
#define APPLICATION_STATES_H


namespace sigviewer
{

enum ApplicationState
{
    APP_STATE_NO_FILE_OPEN,
    APP_STATE_FILE_OPEN
};

}

#endif // APPLICATION_STATES_H
