// Copyright (c) 2016 The SigViewer Development Team
// Licensed under the GNU General Public License (GPL)
// https://www.gnu.org/licenses/gpl


#ifndef FILE_STATES_H
#define FILE_STATES_H

enum FileState
{
    FILE_STATE_UNCHANGED,
    FILE_STATE_CHANGED
};

#endif // FILE_STATES_H
