// Copyright (c) 2016 The SigViewer Development Team
// Licensed under the GNU General Public License (GPL)
// https://www.gnu.org/licenses/gpl


#ifndef COLOR_MANAGER_TESTS_H
#define COLOR_MANAGER_TESTS_H

#include "test.h"

namespace sigviewer
{
namespace Tests_
{
class ColorManagerTests : public Test
{
public:
    ColorManagerTests();
    virtual QString run ();
};
}
}

#endif // COLOR_MANAGER_TESTS_H
